/*
 * Copyright (c) 2006-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: ServiceNotificationSender.java,v 1.16 2007/04/21 18:39:17 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.jython.lib;

import java.net.URI;
import java.util.Iterator;

import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.xmlpull.infoset.XmlElement;

import xsul.ws_addressing.WsaEndpointReference;
import xsul.wsif.WSIFMessage;
import xsul5.MLogger;
import edu.indiana.extreme.lead.workflow_tracking.Notifier;
import edu.indiana.extreme.lead.workflow_tracking.common.InvocationContext;
import edu.indiana.extreme.lead.workflow_tracking.common.InvocationEntity;
import edu.indiana.extreme.lead.workflow_tracking.impl.state.InvocationContextImpl;
import edu.indiana.extreme.xbaya.util.StringUtil;
import edu.indiana.extreme.xbaya.util.XMLUtil;

/**
 * @author Satoshi Shirasuna
 */
public class ServiceNotificationSender {

    private static final MLogger logger = MLogger.getLogger();

    private Notifier notifier;

    private URI workflowID;

    private String nodeID;

    private String serviceID;

    private InvocationEntity receiver;

    private InvocationEntity initiator;

    private InvocationContext invocationContext;

    private WsaEndpointReference eventSink;

    /**
     * Constructs a ServiceNotificationSender.
     * 
     * @param notifier
     * @param eventSink
     * @param initiator
     * @param workflowID
     * @param nodeID
     */
    protected ServiceNotificationSender(Notifier notifier,
            WsaEndpointReference eventSink, InvocationEntity initiator,
            URI workflowID, String nodeID) {
        this.notifier = notifier;
        this.eventSink = eventSink;
        this.initiator = initiator;
        this.workflowID = workflowID;
        this.nodeID = nodeID;
        // In case of creating a service on the fly, there is no serviceID at
        // the beginning.
        this.serviceID = "";

        URI receiverWorkflowID = this.workflowID;
        URI receiverServiceID = URI.create(this.serviceID);
        String receiverWorkflowNodeID = this.nodeID;
        Integer workflowTimeStep = new Integer(0);
        this.receiver = this.notifier.createEntity(receiverWorkflowID,
                receiverServiceID, receiverWorkflowNodeID, workflowTimeStep);
    }

    /**
     * @param serviceID
     */
    public void setServiceID(String serviceID) {
        logger.entering(new Object[] { serviceID });
        this.serviceID = serviceID;

        URI receiverWorkflowID = this.workflowID;
        URI receiverServiceID = URI.create(this.serviceID);
        String receiverWorkflowNodeID = this.nodeID;
        Integer workflowTimeStep = new Integer(0);
        this.receiver = this.notifier.createEntity(receiverWorkflowID,
                receiverServiceID, receiverWorkflowNodeID, workflowTimeStep);
    }

    /**
     * @return The event sink.
     */
    public WsaEndpointReference getEventSink() {
        return this.eventSink;
    }

    /**
     * @return The workflow ID.
     */
    public URI getWorkflowID() {
        return this.workflowID;
    }

    /**
     * @param inputs
     */
    public void invokingService(WSIFMessage inputs) {
        String message = "";
        Iterator partIt = inputs.partNames().iterator();
        boolean first = true;
        while (partIt.hasNext()) {
            if (first) {
                first = false;
            } else {
                message += ", ";
            }
            String name = (String) partIt.next();
            Object value = inputs.getObjectPart(name);
            message += name + "=" + value;
        }
        XmlObject header = null;
        XmlObject body;
        try {
            body = XmlObject.Factory.parse(inputs.toString());
        } catch (XmlException e) {
            logger.warning("Failed to parse " + inputs.toString(), e);
            body = null; // Send notification anyway.
        }
        this.invocationContext = this.notifier.invokingService(this.initiator,
                this.receiver, header, body, message);
    }

    /**
     * @param outputs
     */
    public void serviceFinished(WSIFMessage outputs) {
        String message = "";
        Iterator partIt = outputs.partNames().iterator();
        boolean first = true;
        while (partIt.hasNext()) {
            if (first) {
                first = false;
            } else {
                message += ", ";
            }
            String name = (String) partIt.next();
            Object value = outputs.getObjectPart(name);
            message += name + "=" + value;
        }
        XmlObject header = null;
        XmlObject body;
        try {
            body = XmlObject.Factory.parse(outputs.toString());
        } catch (XmlException e) {
            logger.warning("Failed to parse " + outputs.toString(), e);
            body = null; // Send notification anyway.
        }
        this.notifier.receivedResult(this.invocationContext, header, body,
                message);
    }

    /**
     * Sends an InvokeServiceFinishedFailed notification message.
     * 
     * @param message
     *            The message to send
     * @param e
     */
    public void invocationFailed(String message, Throwable e) {

        // TODO there are two types of error messages.
        // The first one is while creating a service. (No API)
        // The second is while invoking a service.
        // e.g. notifier.invokingServiceFailed().

        // XXX If error occurs before invoking a service, create a fake
        // invocation context.
        if (this.invocationContext == null) {
            this.invocationContext = new InvocationContextImpl(this.initiator,
                    this.receiver);
        }

        logger.entering(new Object[] { message, e });
        logger.caught(e);
        if (message == null || "".equals(message)) {
            message = "Error";
        }
        if (e != null) {
            message += ": " + e.toString();
        }
        if (e != null) {
            String stackTrace = StringUtil.getStackTraceInString(e);
            XmlElement stackTraceElement = XMLUtil.BUILDER
                    .newFragment("stackTrace");
            stackTraceElement.addChild(stackTrace);
            String annotation = XMLUtil.xmlElementToString(stackTraceElement);
            this.notifier.invokingServiceFailed(this.invocationContext, e,
                    message, annotation);
        } else {
            this.notifier
                    .invokingServiceFailed(this.invocationContext, message);
        }
    }

    /**
     * Sends a receivedFault notification message.
     * 
     * @param message
     *            The message to send
     */
    @Deprecated
    public void receivedFault(String message) {
        // XXX If error occurs before invoking a service, create a fake
        // invocation context.
        if (this.invocationContext == null) {
            this.invocationContext = new InvocationContextImpl(this.initiator,
                    this.receiver);
        }

        if (message == null || "".equals(message)) {
            message = "Error";
        }
        this.notifier.receivedFault(this.invocationContext, message);
    }

    /**
     * Sends a receivedFault notification message.
     * 
     * @param fault
     */
    public void receivedFault(WSIFMessage fault) {
        // XXX If error occurs before invoking a service, create a fake
        // invocation context.
        if (this.invocationContext == null) {
            this.invocationContext = new InvocationContextImpl(this.initiator,
                    this.receiver);
        }

        String message = "Received a fault message from the service";
        XmlObject header = null;
        XmlObject body;
        try {
            body = XmlObject.Factory.parse(fault.toString());
        } catch (XmlException e) {
            logger.warning("Failed to parse " + fault.toString(), e);
            body = null; // Send notification anyway.
        }
        this.notifier.receivedFault(this.invocationContext, header, body,
                message);
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2006-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
